public class Traversals {
    private static class Tree<Item> {
        private class Node {
            Item x;
            Node left;
            Node right;

            public Node(Item x, Node left, Node right) {
                this.x = x;
                this.left = left;
                this.right = right;
            }
        }

        Item label;        
        Tree left;
        Tree right;

        public Tree(Item x) {
            label = x;
        }

        public Tree left() {
            return left;
        }

        public Tree right() {
            return right;
        }

        public Item label() {
            return label;
        }
    }


    static String toInfix(Tree<String> T) {
        if (T == null)
            return "";
        if (T.left() == null && T.right() == null)
            return T.label();
        else {
            return String.format("(%s%s%s)", 
                toInfix(T.left()), T.label(), toInfix(T.right()));
        }
    }


    static String toPrefix(Tree<String> T) {
        if (T == null)
            return "";
        if (T.left() == null && T.right() == null)
            return T.label();
        else {
            return String.format("(%s %s %s)", 
                T.label(), toPrefix(T.left()), toPrefix(T.right()));
        }
    }
    static String toPostfix(Tree<String> T) {
        if (T == null)
            return "";
        if (T.left() == null && T.right() == null)
            return T.label();
        else {
            return String.format("(%s %s %s)", 
                toPostfix(T.left()), toPostfix(T.right()), T.label());
        }
    }



    static String toLevel(Tree<String> T) {
        String r = "";
        int H = 3;
        for (int i = 0; i < H; i++) {
                String nextLevel = toLevel(T, i);
                r = r + " " + nextLevel;
            }
        return r;
    }

    static String toLevel(Tree<String> T, int lev) {
        if (T == null)
            return "";
        if (lev == 0)
            return T.label();
        else {
            return String.format("%s %s", 
                   toLevel(T.left(), lev-1),
                   toLevel(T.right(), lev-1));
        }
    }


    public static void print(String s) {
        System.out.println(s);
    }


    public static void main(String[] args) {
        Tree<String> T = new Tree<String>("top");
        T.left = new Tree<String>("AL");
        T.right = new Tree<String>("AR");
        T.left.left = new Tree<String>("BLL");
        T.left.right = new Tree<String>("BLR");

        T.right.left = new Tree<String>("BRL");
        T.right.right = new Tree<String>("BRR");

        System.out.println(toInfix(T));
        System.out.println(toPrefix(T));
        System.out.println(toPostfix(T));
        System.out.println(toLevel(T));
    }

}

